package com.yanzuoguang.util.helper;

import java.util.ArrayList;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class ThreadHelper {
    private static Object _ThreadLock = new Object();
    private static boolean _ThreadIsRun = false;
    private static Date _ThreadDate = null;
    private static ExecuteRun Timeout;
    private static ExecuteRun Interval;

    private static void InitStatic() {
        _ThreadDate = StringHelper.GetDateTime("1987-11-04");
        Timeout = new ExecuteRun();
        Interval = new ExecuteRun();
        ThreadDelegateCode vAdd = new ThreadDelegateCode() {
            public void Execute() {
                OnExecuteAdd();
            }
        };

        ThreadDelegateCode vItemAdd = new ThreadDelegateCode() {
            public void Execute() {
                OnItemExecuted();
            }
        };
        Timeout.OnAdd.Add(vAdd);
        Timeout.OnItemExecuted.Add(vItemAdd);
        Interval.OnAdd.Add(vAdd);
        Interval.OnItemExecuted.Add(vItemAdd);
        StartMonitor();
    }

    // #region 内部累
    private static class ExecuteData {
        public Date Date;
        public double ExecuteCount;
        public double ExecuteError; // 调试变量
        public String Flag;
        public int Time;
        public ThreadDelegateCode Execute;

        public ExecuteData() {
            this.Date = new Date();
        }
    }

    public static class ExecuteRun {
        private Object m_Lock = new Object();
        private ArrayList<ExecuteData> m_List = new ArrayList<ExecuteData>();

        private Event<ThreadDelegateCode> OnAdd = new Event<ThreadDelegateCode>();
        private Event<ThreadDelegateCode> OnRemove = new Event<ThreadDelegateCode>();
        private Event<ThreadDelegateCode> OnItemExecuted = new Event<ThreadDelegateCode>();
        private Event<ThreadDelegateCode> OnExecuted = new Event<ThreadDelegateCode>();

        public Event<ThreadDelegateCode> GetOnAdd() {
            return OnAdd;
        }

        public Event<ThreadDelegateCode> GetOnRemove() {
            return OnRemove;
        }

        public Event<ThreadDelegateCode> GetOnItemExecuted() {
            return OnItemExecuted;
        }

        public Event<ThreadDelegateCode> GetOnExecuted() {
            return OnExecuted;
        }

        /**
         * 执行完任务后，是否需要删除该任务,并且设置允许执行的最大错误
         *
         * @param vIsRemove
         * @param vMaxError
         */
        public final void Execute(boolean vIsRemove, int vMaxError) {
            for (int i = m_List.size() - 1; i >= 0; i--) {
                Date vNow = new Date();
                ExecuteData vItem = null;
                synchronized (this.m_Lock) {
                    vItem = m_List.size() > i ? m_List.get(i) : null;
                }

                if (vItem == null) {
                    continue;
                }

                double vMillseconds = 0;

                // 在Window CE中时间相减可能会出错
                try {
                    // 处理非法改动时间
                    if (vItem.Date.compareTo(vNow) > 0) {
                        vItem.Date = vNow;
                    }

                    vMillseconds = (vNow.getTime() - vItem.Date.getTime());
                } catch (Exception ex) {
                    Log.error(ThreadHelper.class, ex);
                }

                // 未到执行时间
                if (vMillseconds <= vItem.Time) {
                    continue;
                }

                if (vMaxError == -1 || vItem.ExecuteError < vMaxError) {
                    try {
                        vItem.ExecuteCount++;
                        vItem.ExecuteError++;

                        vItem.Execute.Execute();

                        vItem.ExecuteError = 0;

                        this.ExecuteEvent(this.OnItemExecuted);
                    } catch (Exception ex) {
                        Log.error(ThreadHelper.class,ex);
                    }
                    vItem.Date = new Date();
                }

                try {
                    if (vIsRemove) {
                        synchronized (this.m_Lock) {
                            this.m_List.remove(i);
                            this.ExecuteEvent(this.OnRemove);
                        }
                    }
                } catch (Exception ex) {
                    Log.error(ThreadHelper.class,ex);
                }
            }

            this.ExecuteEvent(this.OnExecuted);
        }

        /**
         * 设置需要执行的任务
         *
         * @param vExecute 需要执行的程序
         * @param vTime    间隔时间
         * @return 执行标志
         */
        public final String Set(ThreadDelegateCode vExecute, int vTime) {
            ExecuteData tempVar = new ExecuteData();
            tempVar.Flag = StringHelper.
            tempVar.Execute = vExecute;
            tempVar.Time = vTime;
            ExecuteData vData = tempVar;

            synchronized (m_Lock) {
                m_List.add(vData);
            }

            this.ExecuteEvent(this.OnAdd);

            return vData.Flag;
        }

        /**
         * 清除需要执行的任务
         *
         * @param flag 执行标志
         */
        public final void Clear(String flag) {
            synchronized (m_Lock) {
                for (ExecuteData item : this.m_List) {
                    if (item.Flag.equals(flag)) {
                        this.m_List.remove(item);
                        this.ExecuteEvent(this.OnRemove);
                        break;
                    }
                }
            }
        }

        /**
         * 清除需要执行的任务
         *
         * @param vExecute 执行的方法,注意有多个同样的方法时，只会清除第一个
         */
        public final void Clear(ThreadDelegateCode vExecute) {
            synchronized (m_Lock) {
                for (ExecuteData item : this.m_List) {
                    if (item.Execute == vExecute) {
                        this.m_List.remove(item);
                        this.ExecuteEvent(this.OnRemove);
                        break;
                    }
                }
            }
        }

        private void ExecuteEvent(Event<ThreadDelegateCode> vExecute) {
            try {
                if (vExecute != null) {
                    vExecute.Exeucte(new ThreadDelegateCodeExecute());
                }
            } catch (Exception ex) {
                Log.error(ThreadHelper.class,ex);
            }
        }

        public final int getCount() {
            return this.m_List.size();
        }
    }

    private static void OnItemExecuted() {
        _ThreadDate = new Date();
    }

    private static void OnExecuteAdd() {
        StartThread();
    }

    private static void StartThread() {
        synchronized (_ThreadLock) {
            if (_ThreadIsRun) {
                return;
            }
            RunThread(new ThreadDelegateCode() {
                public void Execute() throws Exception {
                    _ThreadIsRun = true;
                    while (_ThreadIsRun) {
                        try {
                            Timeout.Execute(true, -1);
                        } catch (Exception ex) {
                            Log.error(ThreadHelper.class,ex);
                        }
                        try {
                            Interval.Execute(false, -1);
                        } catch (Exception ex) {
                            Log.error(ThreadHelper.class,ex);
                        }
                        if (Timeout.getCount() == 0 && Interval.getCount() == 0) {
                            break;
                        }
                        Thread.sleep(10);
                    }
                    _ThreadIsRun = false;
                }
            });
        }
    }

    static {
        // InitStatic();
    }

    /**
     * 监控线程的方法，防止线程执行死机
     */
    private static void StartMonitor() {
        RunThread(new ThreadDelegateCode() {
            public void Execute() throws Exception {
                do {
                    if (_ThreadIsRun && ((new Date().getTime() - _ThreadDate.getTime()) / 1000) > 5) {
                        try {
                            if (_ThreadIsRun) {
                                _ThreadIsRun = false;
                            }
                        } catch (Exception ex) {
                            Log.error(ThreadHelper.class,ex);
                        }
                        _ThreadIsRun = false;
                        StartThread();
                    }
                    Thread.sleep(1000 * 60);
                } while (true);

            }
        });
    }

    /**
     * 多少毫秒后执行任务,适用于执行时间短，不需要单独开线程的程序
     *
     * @param vFrom
     *            需要执行的窗体
     * @param vExecute
     *            需要执行的程序
     * @param vTime
     *            间隔时间
     * @return 执行标志
     */
    // public static String SetTimeout(Form vForm, ThreadDelegateCode vExecute,
    // int vTime)
    // {
    // return Timeout.Set(vForm, vExecute, vTime);
    // }

    /**
     * 多少毫秒后执行任务,适用于执行时间短，不需要单独开线程的程序
     *
     * @param vExecute 需要执行的程序
     * @param vTime    间隔时间
     * @return 执行标志
     */
    public static String SetTimeout(ThreadDelegateCode vExecute, int vTime) {
        return Timeout.Set(vExecute, vTime);
    }

    /**
     * 清除需要执行的任务
     *
     * @param flag 执行标志
     */
    public static void ClearTimeout(String flag) {
        Timeout.Clear(flag);
    }

    /**
     * 清除需要执行的任务
     *
     * @param vExecute 执行的方法,注意有多个同样的方法时，只会清除第一个
     */
    public static void ClearTimeout(ThreadDelegateCode vExecute) {
        Timeout.Clear(vExecute);
    }

    /**
     * 每隔多少毫秒执行任务,适用于执行时间短，不需要单独开线程的程序
     *
     * @param vFrom
     *            需要执行的窗体
     * @param vExecute
     *            需要执行的程序
     * @param vTime
     *            间隔时间
     * @return 执行标志
     */
    // public static String SetInterval(Form vForm, ThreadDelegateCode vExecute,
    // int vTime)
    // {
    // return Interval.Set(vForm, vExecute, vTime);
    // }

    /**
     * 每隔多少毫秒执行任务,适用于执行时间短，不需要单独开线程的程序
     *
     * @param vExecute 需要执行的程序
     * @param vTime    间隔时间
     * @return 执行标志
     */
    public static String SetInterval(ThreadDelegateCode vExecute, int vTime) {
        return Interval.Set(vExecute, vTime);
    }

    /**
     * 清除需要执行的任务
     *
     * @param flag 执行标志
     */
    public static void ClearInterval(String flag) {
        Interval.Clear(flag);
    }

    /**
     * 清除需要执行的任务
     *
     * @param vExecute 执行的方法,注意有多个同样的方法时，只会清除第一个
     */
    public static void ClearInterval(ThreadDelegateCode vExecute) {
        Interval.Clear(vExecute);
    }

    public static void Sleep(int p) {
        try {
            Thread.sleep(p);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void ExecuteCatch(ThreadDelegateCode vExecute) {
        try {
            vExecute.Execute();
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.error(ThreadHelper.class,ex);
        }
    }

    private static ExecutorService _ExecuteService = Executors.newCachedThreadPool();

    public static void RunThread(final ThreadDelegateCode vExecute) {
        _ExecuteService.execute(new Runnable() {
            @Override
            public void run() {
                ExecuteCatch(vExecute);
            }
        });
    }

    public static void RunThread(final RunInterval vInterval) {
        RunThread(new ThreadDelegateCode() {
            public void Execute() throws Exception {
                while (!vInterval.getBreak()) {
                    try {
                        vInterval.getCode().Execute();
                    } catch (Exception ex) {
                        Log.error(ThreadHelper.class,ex);
                    }
                    Thread.sleep(vInterval.getTime());
                }
            }
        });
    }

    public static void sleep(long timer) {
        try {
            Thread.sleep(timer);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
